package BaconBird::Model;
use Moose;

use Data::Dumper;

use constant CONSUMER_KEY => "HNWGxf3exkB1mQGpM83PWw";
use constant CONSUMER_SECRET => "dcQsVwycEa6vNCMO0ljbzZfzloqBXcRDMYXRo1bsN7k";

use constant DEFAULT_WAIT_TIME => 60;
use constant MIN_WAIT_TIME => 10;

use constant HOME_TIMELINE => 1;
use constant MENTIONS => 2;
use constant DIRECT_MESSAGES => 3;
use constant SEARCH_RESULTS => 4;
use constant USER_TIMELINE => 5;
use constant HELP => 6;

use Net::Twitter;
use I18N::Langinfo qw(langinfo CODESET);
use Encode::Encoder;

has 'ctrl' => (
	is => 'rw',
	isa => 'BaconBird::Controller',
);

has 'nt' => (
	is => 'ro',
	isa => 'Net::Twitter',
	default => sub { return Net::Twitter->new(traits => [qw/OAuth API::REST InflateObjects RateLimit API::Search/], consumer_key => CONSUMER_KEY, consumer_secret => CONSUMER_SECRET, decode_html_entities => 1, ssl => 1); },
);

has 'user_id' => (
	is => 'rw',
	isa => 'Str',
);

has 'screen_name' => (
	is => 'rw',
	isa => 'Str',
);

has 'home_timeline' => (
	is => 'rw',
	isa => 'ArrayRef',
	default => sub { [ ] },
);

has 'mentions' => (
	is => 'rw',
	isa => 'ArrayRef',
	default => sub { [ ] },
);

has 'direct_messages' => (
	is => 'rw',
	isa => 'ArrayRef',
	default => sub { [ ] },
);

has 'search_results' => (
	is => 'rw',
	isa => 'ArrayRef',
	default => sub { [ ] },
);

has 'user_timeline' => (
	is => 'rw',
	isa => 'ArrayRef',
	default => sub { [ ] },
);

has 'current_timeline' => (
	is => 'rw',
	default => HOME_TIMELINE,
);

has 'home_timeline_ts' => (
	is => 'rw',
	isa => 'Int',
	default => 0,
);

has 'direct_messages_ts' => (
	is => 'rw',
	isa => 'Int',
	default => 0,
);

has 'mentions_ts' => (
	is => 'rw',
	isa => 'Int',
	default => 0,
);

has 'search_results_ts' => (
	is => 'rw',
	isa => 'Int',
	default => 0,
);

has 'user_timeline_ts' => (
	is => 'rw',
	isa => 'Int',
	default => 0,
);

has 'all_messages' => (
	is => 'rw',
	isa => 'HashRef',
	default => sub { { } },
);

has 'all_dms' => (
	is => 'rw',
	isa => 'HashRef',
	default => sub { { } },
);

has 'searchphrase' => (
	is => 'rw',
	isa => 'Str',
);

has 'user_name' => (
	is => 'rw',
	isa => 'Str',
);

sub login {
	my $self = shift;
	my ($at, $ats) = $self->ctrl->load_tokens();
	if ($at && $ats) {
		$self->nt->access_token($at);
		$self->nt->access_token_secret($ats);
	}
	unless ($self->nt->authorized) {
		my $pin = $self->ctrl->get_pin($self->nt->get_authorization_url);
		my ($new_at, $new_ats, $user_id, $screen_name) = $self->nt->request_access_token(verifier => $pin);
		$self->ctrl->save_tokens($new_at, $new_ats);
		$self->user_id($user_id);
		$self->screen_name($screen_name);
	}
}

sub reload_all {
	my $self = shift;
	my $tl = $self->current_timeline;
	if ($tl == HOME_TIMELINE) {
		$self->reload_home_timeline;
	} elsif ($tl == DIRECT_MESSAGES) {
		$self->reload_direct_messages;
	} elsif ($tl == MENTIONS) {
		$self->reload_mentions;
	} elsif ($tl == SEARCH_RESULTS) {
		$self->reload_search_results;
	} elsif ($tl == USER_TIMELINE) {
		$self->reload_user_timeline;
	}
}

sub reload_home_timeline {
	my $self = shift;
	my $id = -1;

	if (defined($self->home_timeline) && scalar(@{$self->home_timeline}) > 0) {
		$id = $self->home_timeline->[0]->{id};
	}

	eval {
		my $newdata = $self->nt->home_timeline({ since_id => $id, count => 50 });
		my $olddata = $self->home_timeline;
		my @new_timeline = ( @$newdata, @$olddata );

		$self->add_new_messages($newdata);

		$self->home_timeline(\@new_timeline);
	};
	if (my $err = $@) {
		die "Reloading home timeline failed: " . $err->error . "\n";
	}
	$self->home_timeline_ts(time);
}

sub reload_mentions {
	my $self = shift;
	my $id = -1;
	if (defined($self->mentions) && scalar(@{$self->mentions}) > 0) {
		$id = $self->mentions->[0]->{id};
	}

	eval {
		my $newdata = $self->nt->mentions({ since_id => $id, count => 50 });
		my $olddata = $self->mentions;
		my @new_mentions = ( @$newdata, @$olddata );

		$self->add_new_messages($newdata);

		$self->mentions(\@new_mentions);
	};
	if (my $err = $@) {
		die "Reloading mentions failed: " . $err->error . "\n";
	}
	$self->mentions_ts(time);
}

sub reload_direct_messages {
	my $self = shift;
	my $id = -1;
	if (defined($self->direct_messages) && scalar(@{$self->direct_messages}) > 0) {
		$id = $self->direct_messages->[0]->{id};
	}

	eval {
		my $newdata = $self->nt->direct_messages({ since_id => $id, count => 50 });
		my $olddata = $self->direct_messages;
		my @new_dms = ( @$newdata, @$olddata );

		$self->add_new_dms($newdata);

		$self->direct_messages(\@new_dms);
	};
	if (my $err = $@) {
		die "Reloading direct messages failed: " . $err->error . "\n";
	}
	$self->direct_messages_ts(time);
}

sub reload_user_timeline {
	my $self = shift;
	my $id = -1;
	if (defined($self->user_timeline) && scalar(@{$self->user_timeline}) > 0) {
		$id = $self->user_timeline->[0]->{id};
	}

	eval {
		my $newdata = $self->nt->user_timeline({ since_id => $id, screen_name => $self->user_name, count => 50 });
		my $olddata = $self->user_timeline;
		my @new_usertl = ( @$newdata, @$olddata );

		$self->add_new_messages($newdata);

		$self->user_timeline(\@new_usertl);
	};
	if (my $err = $@) {
		die "Reloading user timeline failed: " . $err->error . "\n";
	}
	$self->user_timeline_ts(time);
}

sub post_update {
	my $self = shift;
	my ($tweet, $status_id) = @_;

	my $e = Encode::Encoder->new($tweet, langinfo(CODESET()));
	$tweet = $e->iso_8859_1->data;

	$status_id = -1 if !defined($status_id);

	eval {
		$self->nt->update({ status => $tweet, in_reply_to_status_id => $status_id });
	};
	if (my $err = $@) {
		die "Posting update failed: " . $err->error . "\n";
	}
}

sub get_rate_limit {
	my $self = shift;
	return ($self->nt->rate_remaining, $self->nt->rate_limit, $self->nt->rate_reset - time);
}

sub get_wait_time {
	my $self = shift;
	my $waittime = int((($self->nt->rate_reset - time) / $self->nt->rate_remaining) * 1.5);
	$waittime = MIN_WAIT_TIME if $waittime < MIN_WAIT_TIME;
	return $waittime;
}

sub retweet {
	my $self = shift;
	my ($id) = @_;

	if ($self->current_timeline == DIRECT_MESSAGES) {
		die "you can't retweet a direct message.\n";
	}

	eval {
		$self->nt->retweet($id);
	};
	if (my $err = $@) {
		die "Retweet failed: " . $err->error . "\n";
	}
}

sub lookup_author {
	my $self = shift;
	my ($tweetid) = @_;
	my $tweet = $self->get_message_by_id($tweetid);
	return $tweet->{user}{screen_name} if $tweet;
	return "";
}

sub select_timeline {
	my $self = shift;
	my ($timeline) = @_;
	my $old_timeline = $self->current_timeline;
	$self->current_timeline($timeline);
	if ($timeline == HOME_TIMELINE) {
		$self->reload_home_timeline if ($self->home_timeline_ts + DEFAULT_WAIT_TIME) < time;
	} elsif ($timeline == DIRECT_MESSAGES) {
		$self->reload_direct_messages if ($self->direct_messages_ts + DEFAULT_WAIT_TIME) < time;
	} elsif ($timeline == MENTIONS) {
		$self->reload_mentions if ($self->mentions_ts + DEFAULT_WAIT_TIME) < time;
	} elsif ($timeline == SEARCH_RESULTS) {
		if ($self->searchphrase && length($self->searchphrase) > 0) {
			$self->reload_search_results if ($self->search_results_ts + DEFAULT_WAIT_TIME) < time;
		} else {
			$self->current_timeline($old_timeline);
		}
	} elsif ($timeline == USER_TIMELINE) {
		if ($self->user_name && length($self->user_name) > 0) {
			$self->reload_user_timeline if ($self->user_timeline_ts + DEFAULT_WAIT_TIME) < time;
		} else {
			$self->current_timeline($old_timeline);
		}
	}
}

sub get_timeline {
	my $self = shift;
	if ($self->current_timeline == HOME_TIMELINE) {
		return $self->home_timeline;
	} elsif ($self->current_timeline == MENTIONS) {
		return $self->mentions;
	} elsif ($self->current_timeline == DIRECT_MESSAGES) {
		return $self->direct_messages;
	} elsif ($self->current_timeline == SEARCH_RESULTS) {
		return $self->search_results;
	} elsif ($self->current_timeline == USER_TIMELINE) {
		return $self->user_timeline;
	} else {
		# an unknown timeline type is a bug
		return undef;
	}
}

sub add_new_messages {
	my $self = shift;
	my ($msgs) = @_;

	foreach my $m (@$msgs) {
		$self->all_messages->{$m->{id}} = $m;
	}
}

sub add_new_dms {
	my $self = shift;
	my ($msgs) = @_;

	foreach my $dm (@$msgs) {
		$self->all_dms->{$dm->{id}} = $dm;
	}
}

sub get_message_by_id {
	my $self = shift;
	my ($id) = @_;
	return $self->all_messages->{$id};
}

sub get_dm_by_id {
	my $self = shift;
	my ($id) = @_;
	return $self->all_dms->{$id};
}

sub is_direct_message {
	my $self = shift;
	return $self->current_timeline == DIRECT_MESSAGES;
}

sub send_dm {
	my $self = shift;
	my ($tweet, $rcpt) = @_;
	$self->nt->new_direct_message($rcpt, $tweet);
}

sub reload_search_results {
	my $self = shift;
	my $id = -1;
	if (defined($self->search_results) && scalar(@{$self->search_results}) > 0) {
		$id = $self->search_results->[0]->{id};
	}

	eval {
		my $newdata = $self->nt->search({ since_id => $id, q => $self->searchphrase, count => 50 })->{results};
		my $olddata = $self->search_results;
		my @new_search_results = ( @$newdata, @$olddata );

		$self->add_new_messages($newdata);

		$self->search_results(\@new_search_results);
	};
	if (my $err = $@) {
		die "Reloading search results failed: " . $err->error . "\n";
	}
	$self->search_results_ts(time);
}

sub set_search_phrase {
	my $self = shift;
	my ($searchphrase) = @_;
	if (!defined($self->searchphrase) || $searchphrase ne $self->searchphrase) {
		$self->searchphrase($searchphrase);
		$self->search_results([ ]);
		$self->search_results_ts(0);
	}
}

sub get_search_phrase {
	my $self = shift;
	return $self->searchphrase;
}

sub set_user_name {
	my $self = shift;
	my ($user_name) = @_;
	if (!defined($self->user_name) || $user_name ne $self->user_name) {
		$self->user_name($user_name);
		$self->user_timeline([ ]);
		$self->user_timeline_ts(0);
	}
}

sub get_user_name {
	my $self = shift;
	return $self->user_name;
}

sub toggle_favorite {
	my $self = shift;
	my ($tweetid) = @_;
	if ($self->is_direct_message) {
		die "you can't favorite a direct message.\n";
	} else {
		my $tweet = $self->all_messages->{$tweetid};
		if (defined($tweet)) {
			if ($tweet->{favorited}) {
				$self->nt->destroy_favorite($tweetid);
				$tweet->{favorited} = JSON::XS::false;
			} else {
				$self->nt->create_favorite($tweetid);
				$tweet->{favorited} = JSON::XS::true;
			}
		}
	}
}

sub follow_user {
	my $self = shift;
	my ($screen_name) = @_;
	$self->nt->create_friend({ screen_name => $screen_name });
}

sub unfollow_user {
	my $self = shift;
	my ($screen_name) = @_;
	$self->nt->destroy_friend({ screen_name => $screen_name });
}

no Moose;
1;
